#ifndef MyPotNum_h
#define MyPotNum_h

#include <Arduino.h>

#define INTR  0                       // No interrruption 0 => Pin 2 => INT0 pour Arduino nano

  enum Pmode{P_RIEN, P_BUTE, P_WRAPP};
    
    typedef struct Pot{
      int  Compteur;      // compteur utilisable
      int  Compte;        // sous compteur de sensibilité (inverse >0 : moins sensible
      byte Sensible;      // Sensibilité ex: -4 : 4 pas pour un pas de compteur   (V1.02)
                          //                 +10: 1 pas = compteur incrémenté de 10 (V1.03)
      byte Mode;          // voir enum Pmode
      int  Mini;          //+-32000 (avec Min < Max)
      int  Maxi;
    }MY_POT;

class MyPotNum
{
  // 2019 ====================== FONCTIONS ET VARIABLES MEMBRES ====================V1.02==
  //      2019/01  : Adaptation (Pin A,B,BP) carte Equi-Select                             
  //      2020/11  : Adaptation sensibilité pour outil de test allumage z740TC      V1.03
  
  public: 
    int debug;
    
    int sensitive;

    MyPotNum(int);  // Init a la construction de l'objet (parametre d'enttée non utilisé)
  
    /*-------------------------------------------------------------------------*/
    /* Traitement du Comptage du potentiometre numerique (passe par adresse)   */
    /*     (En liaison avec IT potentiometre)                                  */
    /*  retour TRUE si un traitement execute (inc ou dec du compteur)          */
    /*  peut etre placé dans la tache de fond ou appelé cycliquement...        */
    /*-------------------------------------------------------------------------*/
    boolean GereComptage(MY_POT *);

    /*-------------------------------------------------------------------------*/
    /* Retourne l'etat du BP                                                   */
    /*-------------------------------------------------------------------------*/
    boolean BpPushed(void);
    
    /*-------------------------------------------------------------------------*/
    /* Retourne l'etat du du sens de rotation                                  */
    /*-------------------------------------------------------------------------*/
    boolean Up(void);
   

  //======================== VARIABLES ET FCT LOCALES ET PRIVEES ====================
  private:   
  
    //-------- Definition des pinoches utiles : 
    static const byte interruptPinPhaseA = 2;    // ATTENTION Interrution physique INT0
    static const byte PinPhaseB = 3;     // Pour testeur allumage 740 TC (V1.03)
     
    #define USE_BP_A7   
    static const int PinBP = A7;      // Modif V1.02
    //static const byte PinBP = 10;
    
    //---------Les variables utiles au traitement entres IT et fonctions...
    static MyPotNum* ptc;     // pointeur de contexte sur l'objet
    volatile byte flagUpDown;
    volatile byte countUP;
    volatile byte waitRisingPhase;
    
    
    /*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    /* Traite interruption du potentiometre incremental sur ce principe :      */
    /*    1ere etape : Phase A declenche IT sur front descendant               */
    /*       - determine en fontion de la phase B le sens UP ou DWN            */
    /*       - reprogramme IT sur detection front montant.                     */
    /*    2eme etape :  sur IT confirmation                                    */
    /*       - confirmation en fontion de la phase B du sens UP ou DWN         */
    /*       - monte un flag pour traitement dans tache de fond. (ou IT timer) */
    /*       - reprogramme IT sur detection front descendant.                  */
    /*                                                                         */
    /* QQ chose a traiter par tache de fond si "flagUpDown = 1"                */
    /*    dans ce cas "countUP"= 1 ou 0 en foction du sens                     */
    /*    Puis la tache de fond acquite (flagUpDown=0)                         */
    /*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    static void IT_PotNum(void);

};
#endif
