
/* VERSION FONCTIONNELLE
 *  
 *     Version testeur pour  allumage electronique pour Kawa z740-TC
 *     10_2020 
 *          V1.10 Reprise sur nano pour Projet TCI_z740 (650 Turbo)
 *            - ajout sortie pour 2 eme rupteur pour 4 Cylindres (D12 et D13)
 *          V1.20 Ajout affichage LCD 
 *          V1.30 réécriture pour plus de précision.
 *                -ajout d'un changement de base de temps plage de 5ms a 8s
 *                -ajout seleteur incrémental rotatif
 *                -ajout gestion de mode de test : BP et inter 
 *                    mode normal reglage avec le pot D ou C par Dixaine ou Centaine
 *                    Inter basculé et appui sur bp pot: mode 1 panne capteur 1
 *                                                       mode 2 panne capteur 2
 *                                                       mode * panne des 2 capteurs
 *                                                       mode / rampe automatique 
 *          V1.31 Ajout    mode "z" rampe sur transition Z1-Z2 (720tr) pour débug
 *                         mode "Z" rampe sur transition Z2-Z3 (3790tr) pour débug
 *          
 * Timer1 : 16 bis pour calcul fréquence rotation et calcul du temps d'avance.
 *
 *   Pulse 5ms a 8s sur sortie 13 (256µs)
 *   Pulse 5ms a 8s sur sortie 12 (256µs) décalé a 180°
 *   
 *   Pot Numérique   
 * CLK A sur 8
 * CLK B sur 9
 *   
 *    Pour raccorder le LCD :
 * LCD RS pin to digital pin 9
 * LCD Enable pin to digital pin 8
 * LCD D4 pin to digital pin 7
 * LCD D5 pin to digital pin 6
 * LCD D6 pin to digital pin 5
 * LCD D7 pin to digital pin 4
 * LCD R/W pin to ground
 * 10K resistor:
 * ends to +5V and ground
 * wiper to LCD VO pin (pin 3)
 * 
 HIPONEY 2020 */
 
//http://fp.fifou.free.fr

//----------- include the library code:--------------

#include <LiquidCrystal.h>
#include "MyPotNum.h"

#define S_Ignition_One  13                  // Sortie commande 1-4
#define S_Ignition_Two  12                  // Sortie commande 2-3
#define E_Mode0         11                  // Inter Mode 
#define E_Mode1         10                  // utilisé pour BP potentiometre

#define dureeAntiRebond 10                  //ms pour codeur
     
//const int analogInPin = A0;               // Analog input pin that the potentiometer is attached to
                        
#define CYCLE_AFFICHAGE 500                 // affichage toutes les 500ms

#define MAX_TIME  62500                     // max timer1 250ms
#define MIN_TIME  625                       // max timer1 2.5ms 
#define PULSE    256                        // 50 * 4µs = 256µs  (2^)                 

unsigned short valeur_Top = PULSE>>2;       // valeur top en valeur comptage

unsigned long Periode = 60000;              // Valeur Periode en µs (1000tr/min)       
unsigned long PeriodeMem = 0;               // mem pour détection de changement
unsigned short valeur_Demi_Periode;         // valeur 1/2 periode en valeur comptage

enum{TOP_1_4,ALL_1_4,TOP_2_3,ALL_2_3};
unsigned short phase = TOP_1_4;             // Phase du séquenceur d'it

unsigned short echelle = 4;                 // 1/16MHz * 64 = 4µs 
enum{TimerOff, _4_us, _16_us, _64_us};

//---Gestion du pot  
long ValPotNumerique=1000; 
long MemPotNumerique=1000;              
unsigned long VitPotNumerique=0;              

unsigned char modeDef;

//---------------- Pour LCD ------------------

// initialize the library by associating any needed LCD interface pin
// with the arduino pin number it is connected to
const int rs = 9, en = 8, d4 = 7, d5 = 6, d6 = 5, d7 = 4;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);

// these constants won't change.  But you can change the size of
// your LCD using them:
const int numRows = 2;
const int numCols = 16;

//----------- Pour Potentiometre Codeur quadrature ---------
static MY_POT  Pot;
MyPotNum PotReglage(0);    //Pour Bouton codeur selecteur de menus
  
/*--------------------------------------------------------------*/
/*  INITIALISATIONS                                             */
/*--------------------------------------------------------------*/
void setup(){

  // set up the LCD's number of columns and rows:
  lcd.begin(numCols, numRows); 

  // initialize serial communications at 9600 bps:
  Serial.begin(9600); 
  
  // set pins as outputs
  pinMode(S_Ignition_One, OUTPUT);                   // Sortie Vis 1_4
  pinMode(S_Ignition_Two, OUTPUT);                   // Sortie Vis 2_3
  
  pinMode(E_Mode0,INPUT_PULLUP);                    // Selection mode
  pinMode(E_Mode1,INPUT_PULLUP);                    // Selection mode
  
  //---------- Init Potentiomètre numérique ------------
  Pot.Compteur=1000;    // compteur utilisable
  Pot.Compte=0;         // sous compteur de sensibilité
  Pot.Sensible=100;     // pas de sensibilité ex: 4  compteur= compteur + 4 et-4  diminue la sensibilité : 4  TI pour un pas de de compteur
  Pot.Mode=P_BUTE;      // voir enum Pmode : gestion de butté
  Pot.Mini=10;         // def buttés
  Pot.Maxi=10000;
  
  lcd.setCursor(15, 0);              
  lcd.print("C"); //  cetaine par defaut pour sensibilité
 
  phase = TOP_1_4;  // Init séquenceur IT
  
  //---- Timer1 16 bits TCCR division par 64  time 4µs à 262ms (de base)
  cli(); //stop interrupts
  
  TCCR1A = 0; // TCCR1A Timer counter control register to 0
  TCCR1B = 0; // TCCR1B
  
  TCNT1  = 0; // Initialize counter value to 0
  
  // set compare match register for 1hz increments
  OCR1A = valeur_Top; // = (16*10^6) / (0,1s *div64) - 1 (must be <65536)
  
  // turn on CTC mode
  TCCR1B |= (1 << WGM12);
  
  // Set CS12 and CS10 bits for 64 prescaler
  TCCR1B |= (1 << CS11) | (1 << CS10);  
  
  // enable timer compare interrupt
  TIMSK1 |= (1 << OCIE1A);

  sei();//allow interrupts
  
  ReglageDeLaPeriode(Periode);    // reglage du timer
  
}//end setup


//----------------------------------------------
//  Pour changer la Période
//
//    Entrée de 8s a 5ms (en µs)
//      - calcul la valeur_Demi_Periode
//      - Change la base de temps
//----------------------------------------------
void ReglageDeLaPeriode(unsigned long periode)
{ 
  //-----Test des limites de fonctionnement----
  if(periode < 5000)periode = 5000;
  if(periode > 8000000)periode = 8000000;

  //------ Selection et calcul de valeur ------
  if (periode < 500000 )                              //  SI < 500ms div/64 = 4µs
  {
    valeur_Demi_Periode =((periode>>2)-(PULSE>>1))>>1;      // valeur = ((periode/4µs) - (2*PULSE/4µ))/2
    valeur_Top = PULSE>>2;                                  // valeur top en valeur comptage /4µs
    echelle = ChangeBaseDeTemps(_4_us);
  }
  else if (periode < 2000000)                         // SI entre 500ms et 2s : div/256 = 16µs
  {
    valeur_Demi_Periode =((periode>>4)-(PULSE>>3))>>1;      // valeur = ((periode/16µs) - (2*PULSE/16µ))/2 
    valeur_Top = PULSE>>4;                                  // valeur top en valeur comptage /16µs
    echelle = ChangeBaseDeTemps(_16_us);
  }
  else                                                // SINON div/1024 = 64µs de 2 a 8s
  {
    valeur_Demi_Periode =((periode>>6)-(PULSE>>5))>>1;      // valeur = ((periode/64µs) - (2*PULSE/64µ))/2     
    valeur_Top = PULSE>>6;                                  // valeur top en valeur comptage /64
    echelle = ChangeBaseDeTemps(_64_us);
  }  
}


//----------------------------------------------
//  Pour changer la base de temps du timer 1
//----------------------------------------------
unsigned char ChangeBaseDeTemps(unsigned char base)
{
  
  cli();
  switch (base)
  {
    case TimerOff:  
      TCCR1B &= ~((1 << CS12) | (1 << CS11) | (1 << CS10));    // CS12, CS11, CS10 = 0 0 0
      sei();      
      return 0; 
    
    case _4_us:     // Division par 64 du 16MHz
      TCCR1B &= ~(1 << CS12);  
      TCCR1B |=  (1 << CS11) | (1 << CS10);                    // CS12, CS11, CS10 = 0 1 1
      sei();      
      return 4; 
    
    case _16_us:    // Division par 256 du 16MHz
      TCCR1B &=  ~((1 << CS11) | (1 << CS10));                 // CS12, CS11, CS10 = 1 0 0  
      TCCR1B |=   (1 << CS12);    
      sei();      
      return 16; 
    
    case _64_us:    // Division par 1024 du 16MHz
      TCCR1B &=  ~(1 << CS11);  
      TCCR1B |=  ((1 << CS12) | (1 << CS10));                 // CS12, CS11, CS10 = 1 0 1     
      sei();      
      return 64; 
      
  }
}

//+++++++++++++++++++++++++++++++++++++++++++++ 
// timer1 interrupt sur registre comparaison A
//+++++++++++++++++++++++++++++++++++++++++++++
ISR(TIMER1_COMPA_vect)
{
  cli();
  
  TCNT1=0;  // Raz T1 

  switch (phase)
  {
    case TOP_1_4 :
      if ((modeDef&1)!=1) 
          digitalWrite(S_Ignition_One,HIGH);              // en fonction de la selection on coupe les sorties pour test
      OCR1A = valeur_Top ;                                // TopImpulsion  calibrée 
      phase = ALL_1_4;
    break;
    
    case ALL_1_4:
      digitalWrite(S_Ignition_One,LOW);
      OCR1A = valeur_Demi_Periode;                        // valeur 1/2 periode
      phase = TOP_2_3;
    break;
    
    case TOP_2_3 :
      if ((modeDef&2)!=2) 
          digitalWrite(S_Ignition_Two,HIGH);
      OCR1A = valeur_Top;                                 // TopImpulsion  calibrée 
      phase = ALL_2_3;
    break;
    
    case ALL_2_3:
      digitalWrite(S_Ignition_Two,LOW);
      OCR1A = valeur_Demi_Periode;                        // valeur 1/2 periode
      phase = TOP_1_4;
    break;
  }
  sei();
}



//====================================================================================
// Tache de fond : Affichage et gestion de la base de temps
//====================================================================================
void loop()
{
  
 static unsigned short rpm,PotRPM,LimH,LimL,Vite;
 static unsigned long temp,milli_s, milli_s_sur_dix, micro_s;
 static unsigned char modeDigit=0,modePrec=0,RPMsens=0;
 static unsigned long currentMillis,previousAffichage=0;
 
   currentMillis = millis();        // Lecture du temps systeme
    
   //--------- Gestion Detection changement mode Inter + BP pot ----------   
 
   if (digitalRead(E_Mode0)==0)             //  lecture BP (actif =0)
   {
      if (digitalRead(E_Mode1))             // si inter : autre fonctions : couper un des capteur
      {
        if (modeDef<4)modeDef++; else modeDef<<=1;
        if (modeDef>16) modeDef=0;           //  selecteur Max       
      }else{
        if (modeDigit) modeDigit=0;  else modeDigit=1;     //  bascule 
      }
      delay(5);
      while (digitalRead(E_Mode0)==0);
   }
   
   //---------------- Selection Digit --------------
   if ((modeDigit+modeDef)!=modePrec)   //  traitement du mode si changement
   {
     //------ Partie digit ------ 
     lcd.setCursor(15, 0);              //  colonne, rang 
     if (modeDigit)
     {
        Pot.Sensible=10;
        lcd.print("D");
     }else{
        Pot.Sensible=100;
        lcd.print("C");
     }
     
     //------ ou Partie Defaut sortie rampe etc...------ 
     lcd.setCursor(14, 0);              //  colonne, rang 
     
     switch (modeDef) 
     {
     case 1:
        lcd.print("1"); break;
     case 2:
        lcd.print("2"); break;
     case 3:
        lcd.print("*"); break;
     case 4:     // Rampe ralenti ... Rone rouge
        LimH=9500;
        LimL=800;
        PotRPM=LimL;
        Vite=900;
        lcd.print("/"); break;
     case 8:     // Rampe entre Z1 et Z2
        LimH=1000;
        LimL=400;
        PotRPM=LimL;
        Vite=1500;
        lcd.print("z"); break;
     case 16:    // Rampe entre Z2 et Z3
        LimH=4500;
        LimL=3500;
        PotRPM=LimL;
        Vite=1500;
        lcd.print("Z"); break;
     default:
        lcd.print(" "); break;
     }  
    
     modePrec=modeDigit+modeDef;
   }
   
  //---------------------- Gestion du Bouton potentiometre ou RAMPE Automatique -------------------------  
  if ( modeDef<4 )                        // si > 3 on est en mode test rampe auto
  {
    //------------- Mode normal prise en compte du potentiomètre numerique -------------
    if (PotReglage.GereComptage(&Pot))    // Passe l'adresse d'un entier compteur pour potentiometre 
    {   
  
      PotRPM = Pot.Compteur;              // Lecture pot numérique
  
      //---- calcul de la période en fonction de la valeur RPM ------     
      Periode = 60000000L / PotRPM;       // Periode correpondante en µs
       
      if(Periode != PeriodeMem )          //détection de changement période
      {
          PeriodeMem = Periode;
          ReglageDeLaPeriode(Periode);    // reglage du timer
      }
    }
  }else{
      //----------------------------Rampes automatiques ----------------------
      
      //------------- Rampe generation -------------
      if (RPMsens==0)
      {  
        PotRPM++;
        if (PotRPM>=LimH) RPMsens=1;
      }else{
        PotRPM--;
        if (PotRPM<=LimL) RPMsens=0;
      }
    
      //---- calcul de la période en fonction de la valeur RPM ------    
      Periode = 60000000L / PotRPM;       // Periode correpondante en µs
       
      if(Periode != PeriodeMem )          //détection de changement période
      {
          PeriodeMem = Periode;
          ReglageDeLaPeriode(Periode);    // reglage du timer
      }
      delayMicroseconds(Vite); 
  }

   
  //-------------- Est-il temps d'afficher sur le LCD ? ---------------
  
  if ((currentMillis - previousAffichage) >= CYCLE_AFFICHAGE) 
  {
     previousAffichage = currentMillis;

     //valeur = constrain(analogRead(analogInPin), MIN_TIME, MAX_TIME);           
   
     temp = ((unsigned long)valeur_Demi_Periode*2)+((unsigned long)valeur_Top*2);
     milli_s = ((temp*echelle)/1000);   
     milli_s_sur_dix = ((temp*echelle)/100);   
     micro_s = (temp*echelle); 
      
     rpm=(unsigned short)(60000000L/(temp*echelle));
 /*    
     // print de DEBUG 
     Serial.print(Periode);
     Serial.print("µs  demi-p:");
     Serial.print(valeur_Demi_Periode);
     Serial.print("pts ");

     Serial.print(echelle);
     Serial.print("eµs ");                     
     Serial.print(temp);      
     Serial.print(" pts\t");
     Serial.print(micro_s);
     Serial.print("µs  ");
     Serial.print(" pot:");
     Serial.print(PotRPM);
     Serial.print(" mode:");
     Serial.print(Pot.Sensible);
     Serial.print('\n');
 */  
      
     // info sur LCD
     lcd.setCursor(0, 0);  //colonne, rang  
     lcd.print(echelle);
     lcd.print("us ");

     lcd.setCursor(5, 0);  //colonne, rang  
     lcd.print("RPM:     ");
     lcd.setCursor(9, 0);  //colonne, rang  
     lcd.print(rpm/10);
     lcd.print("0 ");
          
     lcd.setCursor(0, 1);  //colonne, rang  
     lcd.print("T:              ");      
     lcd.setCursor(2, 1);  //colonne, rang  
     lcd.print(temp);      
       
     lcd.setCursor(9, 1);  //colonne, rang  
     lcd.print(milli_s);      
     lcd.print(",");
     lcd.print(milli_s_sur_dix-(milli_s*10));      
     lcd.print("ms");
  }               
}
